@@ -37,7 +37,7 @@
 /*
  * CMS recipientInfo methods.
  *
- * $Id: cmsrecinfo.c,v 1.16 2005/10/03 22:01:57 relyea%netscape.com Exp $
+ * $Id: cmsrecinfo.c,v 1.16.2.1 2006/03/03 04:03:39 nelson%bolyard.com Exp $
  */
 
 #include "cmslocal.h"
@@ -295,6 +295,9 @@ done:
     return ri;
 
 loser:
+    if (ri && ri->cert) {
+        CERT_DestroyCertificate(ri->cert);
+    }
     if (freeSpki) {
       SECKEY_DestroySubjectPublicKeyInfo(freeSpki);
     }
