@@ -34,7 +34,7 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
-/* $Id: keydb.c,v 1.40.2.1 2006/04/04 18:44:18 wtchang%redhat.com Exp $ */
+/* $Id: keydb.c,v 1.40.2.3 2006/05/17 17:56:37 alexei.volkov.bugs%sun.com Exp $ */
 
 #include "lowkeyi.h"
 #include "seccomon.h"
@@ -713,7 +713,6 @@ nsslowkey_UpdateKeyDBPass1(NSSLOWKEYDBHandle *handle)
     DBT key;
     DBT data;
     unsigned char version;
-    SECItem *rc4key = NULL;
     NSSLOWKEYDBKey *dbkey = NULL;
     NSSLOWKEYDBHandle *update = NULL;
     SECItem *oldSalt = NULL;
@@ -882,10 +881,6 @@ done:
 
     nsslowkey_CloseKeyDB(update);
     
-    if ( rc4key ) {
-	SECITEM_FreeItem(rc4key, PR_TRUE);
-    }
-    
     if ( oldSalt ) {
 	SECITEM_FreeItem(oldSalt, PR_TRUE);
     }
@@ -938,7 +933,7 @@ openNewDB(const char *appName, const char *prefix, const char *dbname,
      * local database we can update from.
      */
     if (appName) {
-        NSSLOWKEYDBHandle *updateHandle = nsslowkey_NewHandle(updatedb);
+        NSSLOWKEYDBHandle *updateHandle;
 	updatedb = dbopen( dbname, NO_RDONLY, 0600, DB_HASH, 0 );
 	if (!updatedb) {
 	    goto noupdate;
